# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import time


def delay(secs: float) -> generator:
    end = time.monotonic() + secs
    while time.monotonic() < end:
        yield None


def fps(fps: float) -> generator:
    yield None
    dt = 1 / fps
    until = time.monotonic() + dt
    while True:
        remain = until - time.monotonic()
        if remain > 0:
            time.sleep(remain)
        until += dt
        if remain < -dt:
            # Catch up a bit
            until += -remain / 3
        yield None


def wait(stop: int) -> generator:
    start = time.monotonic()
    while True:
        elapsed = int((time.monotonic() - start) * 1000)
        elapsed = min(stop, elapsed)
        yield elapsed
        if elapsed >= stop:
            break


def test():
    for f in delay(2.0):
        print(utime.ticks_ms())
